clc
clear all
close all
format long

% Construction of exponential sums of good approximation for the 
% Dirichlet kernel - Example 6.3 of the paper 
% Derevianko, Plonka, Petz: From ESPRIT to ESPIRA: Estimation of  Signal
% Parameters by Iterative Rational Approximation

% use L=1000 for MPM and for ESPRIT
% Use modified ESPIRA-II with Cauchy matrices instead of Vandermonde matrices in ESPIRA II file espira2appr for computation of
% coefficients g (lines 82 - 88 instead of 71-80)

Nexp=44; % order of the constructed exponential sum
N=1000;
L=1000;

b=1;
sf=1/(2*N);
x=0.001:0.001:1-0.001;

% computation of function values
y=[];
for j=1:2*N
     y=[y; dirich2(sf*(j-1)*b,50)]; 
end

[g,pol]=espira1appr(y,Nexp);   % espira I
[g1,pol1]=mpmappr(y,Nexp,L);     % mpm 
[g2,pol2]=espira2apprmod(y,Nexp);  % espira II (modified)
[g3,pol3]=espritappr(y,Nexp,L);  % esprit


a=log(pol);
a1=log(pol1);
a2=log(pol2);
a3=log(pol3);
Ns=length(pol2)


yd=dirich2(b*x,50);


yes1=statio(g,a,2*N*x);
ympm=statio(g1,a1,2*N*x);
yes2=statio(g2,a2,2*N*x);
yespr=statio(g3,a3,2*N*x);

disp('errors in maximum norm')
error_espira1=norm(yd-yes1,inf)
error_mpm=norm(yd-ympm,inf)
error_esprit=norm(yd-yespr,inf)
error_espira2=norm(yd-yes2,inf)

figure
abb=plot(x,yd);
set(abb,'linewidth',2); axis([0,1,-0.3,1.1]); set(gca,'fontsize',18); print -depsc bsp22.eps;
title('Dirichlet kernel')

figure
abb=plot(x, log10(abs(yd-yes1)));
set(abb,'linewidth',2); axis([0,1,-10.5,-8.7]); set(gca,'fontsize',18); print -depsc bsp22.eps;
title('Error for ESPIRA I')

figure
abb=plot(x, log10(abs(yd-ympm)));
set(abb,'linewidth',2); axis([0,1,-4.5,0]); set(gca,'fontsize',18); print -depsc bsp22.eps;
title('Error for MPM')

figure
abb=plot(x, log10(abs(yd-yes2)));
set(abb,'linewidth',2); axis([0,1,-7.5,-6.6]); set(gca,'fontsize',18); print -depsc bsp22.eps;
title('Error for ESPIRA II')

figure
abb=plot(x, log10(abs(yd-yespr)));
set(abb,'linewidth',2); axis([0,1,-4.5,0]); set(gca,'fontsize',18); print -depsc bsp22.eps;
title('Error for ESPRIT')



figure
axis([0 1.2 -1 1])
pos = [-1 -1 2 2]; 
rectangle('Position',pos,'Curvature',[1 1])
hold on
plot(pol,'+','MarkerSize',12)
hold on
pos = [-1 -1 2 2]; 
rectangle('Position',pos)
title('Knots computed with ESPIRA I')
hold off

figure
pos = [-0.15 -0.15 0.3 0.3]; 
rectangle('Position',pos,'Curvature',[1 1])
axis equal
hold on
plot(g,'+','MarkerSize',12)
hold on
pos = [-0.15 -0.15 0.3 0.3]; 
rectangle('Position',pos)
title('Weights computed with ESPIRA I')
hold off



function f=statio(g1,a1,x)
[N,~]=size(g1);
f=0;
for k=1:N
f=f+g1(k)*exp(a1(k)*x);
end
end


function f=dirich2(x,n)
if x==0
    f = 1;
elseif x==1
    f=1;
else
f = sin((2*n+1)*pi*x)./((2*n+1).*sin(pi*x));
end
end

