clc
clear all
close all
format long

% Experiments for exact data
% Examples 5.1-5.3 in
% Derevianko, Plonka, Petz: From ESPRIT to ESPIRA: Estimation of  Signal
% Parameters by Iterative Rational Approximation



%input signal
input_s=3;

if input_s==1              %Example 5.1
    K=6;
    g=[1; 2; 3; 4; 5; 6];
    knots=[0.9856-0.1628*1i; 0.9856+0.1628*1i; 0.8976-0.4305*1i; 0.8976+0.4305*1i; 0.1827-0.5690*1i; 0.1827+0.5690*1i];
elseif input_s==2          %Example 5.2
    K=6;
    a=[7*1i/1000; 21*1i/1000; 200*1i/1000; 201*1i/1000; 53*1i/1000; 1i];
    g=[6; 5; 4; 3; 2; 1];
    knots=exp(a);
    tol=10^(-13);
elseif input_s==3          %Example 5.3
    K=6;
    a=[200*1i/1000; 201*1i/1000; 202*1i/1000; 203*1i/1000; 204*1i/1000; 205*1i/1000];
    g=[6; 5; 4; 3; 2; 1];
    knots=exp(a);
    tol=10^(-13);
end

 
% computation of function values
L=400; % upper bound for K
N=400; % number of points 2N
y=[];
for j=1:2*N
    y=[y; statio(g,knots,(j-1))];   
end

x=0:0.001:2*N-1;


[M1,pol1,g1]=esprit(y,L);          % ESPRIT 
[M2,pol2,g2]=matrix_pencil(y,L);   % MPM  
[M3,pol3,g3]=espira1(y);           % ESPIRA I
[M4,pol4,g4]=espira2(y,tol);       % ESPIRA II


y=statio(g,knots,x); % original exponential sum

% results for ESPRIT
disp('                     results for ESPRIT');
y1=statio(g1,pol1,x); % exponential sum reconstructed with ESPRIT
errorf=norm(y-y1,inf)./max(y);
disp('error function');
disp(errorf);

%if M1>K knots=[knots; zeros(M1-K,1)]; end;
knotsreal=sort(real(knots));
knotsimag=sort(imag(knots));


pol1real=sort(real(pol1));
pol1imag=sort(imag(pol1));
errorknotsreal=norm(knotsreal-pol1real,inf)./max(abs(knots));
errorknotsimag=norm(knotsimag-pol1imag,inf)./max(abs(knots));
%errorknots=norm(knots-pol1,inf)./max(abs(knots));
errorknots=max(errorknotsreal,errorknotsimag);
disp('error knots');
disp(errorknots);

%if M1>K g=[g; zeros(M1-K,1)]; end;
g1=sort(g1);
g=sort(g);
errorg=norm(g-g1,inf)./max(abs(g));
disp('error coefficients');
disp(errorg);

% results for MPM
disp('                     results for MPM');
y2=statio(g2,pol2,x); % exponential sum reconstructed with MPM
errorf=norm(y-y2,inf)./max(y);
disp('error function');
disp(errorf);

%if M2>K knots=[knots; zeros(M2-K,1)]; end;
pol2real=sort(real(pol2));
pol2imag=sort(imag(pol2));
errorknotsreal=norm(knotsreal-pol2real,inf)./max(abs(knots));
errorknotsimag=norm(knotsimag-pol2imag,inf)./max(abs(knots));
errorknots=max(errorknotsreal,errorknotsimag);
disp('error knots');
disp(errorknots);

%if M2>K g=[g; zeros(M2-K,1)]; end;
g2=sort(g2);
g=sort(g);
errorg=norm(g-g2,inf)./max(abs(g));
disp('error coefficients');
disp(errorg);

% results for ESPIRA I
disp('                     results for ESPIRA I');
y3=statio(g3,pol3,x); % exponential sum reconstructed with ESPIRA I
errorf=norm(y-y3,inf)./max(y);
disp('error function');
disp(errorf);

%if M3>K knots=[knots; zeros(M3-K,1)]; end;
pol3real=sort(real(pol3));
pol3imag=sort(imag(pol3));
errorknotsreal=norm(knotsreal-pol3real,inf)./max(abs(knots));
errorknotsimag=norm(knotsimag-pol3imag,inf)./max(abs(knots));
errorknots=max(errorknotsreal,errorknotsimag);
disp('error knots');
disp(errorknots);


%if M3>K g=[g; zeros(M3-K,1)]; end;
g3=sort(g3);
g=sort(g);
errorg=norm(g-g3,inf)./max(abs(g));
disp('error coefficients');
disp(errorg);

% results for ESPIRA II
disp('                     results for ESPIRA II');
y4=statio(g4,pol4,x); % exponential sum reconstructed with ESPIRA II
errorf=norm(y-y4,inf)./max(y);
disp('error function');
disp(errorf);

%if M4>K knots=[knots; zeros(M4-K,1)]; end;
pol4real=sort(real(pol4));
pol4imag=sort(imag(pol4));
errorknotsreal=norm(knotsreal-pol4real,inf)./max(abs(knots));
errorknotsimag=norm(knotsimag-pol4imag,inf)./max(abs(knots));
errorknots=max(errorknotsreal,errorknotsimag);
disp('error knots');
disp(errorknots);


%if M4>K g=[g; zeros(M4-K,1)]; end;
g4=sort(g4);
g=sort(g);
errorg=norm(g-g4,inf)./max(abs(g));
disp('error coefficients');
disp(errorg);


function f=statio(g1,knot1,x)
[N,~]=size(g1);
f=0;
for k=1:N
f= f+ g1(k)*(knot1(k)).^x;
end
end


