clc
clear all
close all
format long

% Experiments for noisy data, Section 5 in 
% Derevianko, Plonka, Petz: From ESPRIT to ESPIRA: Estimation of Signal
% Parameters by Iterative Rational Approximation
 
%Example 5.4 and 5.5
 K=8;
 a=[11*1i/1000; 21*1i/1000; 23*1i/1000; 203*1i/1000; 205*1i/1000; 279*1i/1000; 553*1i/1000; 1i];
 g=[4; 5; 4; 3; 2; 1; 2; 3];
 knots=exp(2*pi*a);
 
%Choice of noise
 input_n=2;
 
 if input_n==1         %uniform noise on the interval [-10,10]
 elseif input_n==2     %Gaussian noise
    rho =2;           % Signal to noise ratio
 end

L=800; % 2*L is the number of used (noisy) signal values
anz=2; % number of tests 

% computation of function values
ra=[];
for j=1:2*L
    ra=[ra; statio(g,knots,j-1)];   
end

a1=a;  
g1=g; 
if input_n==2
    sig = std(ra(:));
    sigma = sig / rho;   %Compute sigma for Gaussian noise
end
%initializations
errorf=zeros(anz,1);
errorfm=zeros(anz,1);
errorrealknots_ESPIRA=zeros(anz,1);
errorimagknots_ESPIRA=zeros(anz,1);
errorrealknots_MPM=zeros(anz,1);
errorimagknots_MPM=zeros(anz,1);
errorg=zeros(anz,1);
errorgm=zeros(anz,1);
PSNR=zeros(anz,1);
SNR=zeros(anz,1);

for j=1:anz
  if input_n==1  
  f= ra+20.0*(rand(length(ra),1)-0.5); %add uniform noise
  else
  f = ra + sigma * randn(size(ra)); %add Gaussian noise
  end
  
err0=f-ra;
PSNR(j)=10*log10(max(ra)^2/mean(err0(:).^2));
SNR(j) = 10 * log10(var(ra(:), 1) / mean(err0(:).^2));

tic
[g,pol]=espira2appr(f,K); % Espira II
toc
[gm,pol1]=mpmappr(f,K);   % MPM
toc

x=0:0.001:2*L-1;
y=statio(g1,knots,x); % original exponential sum

% computation of errors for ESPIRA II
disp('Results obtained with ESPIRA II iteration ...');
disp(j);

y1=statio(g,pol,x);   % exponential sum reconstructed with ESPIRA II
errorf(j)=norm(y-y1,inf)./max(y);
disp('error function');
disp(errorf(j));

realknots=sort(real(knots));
realpol=sort(real(pol));
imagknots=sort(imag(knots));
imagpol=sort(imag(pol));
errorrealknots_ESPIRA(j)=norm(realknots-realpol,inf);
errorimagknots_ESPIRA(j)=norm(imagknots-imagpol,inf);
disp('error knots (real part and imaginary part)');
disp(errorrealknots_ESPIRA(j));
disp(errorimagknots_ESPIRA(j));

gs=sort(g);
g1s=sort(g1);
errorg(j)=norm(gs-g1s,inf)./max(abs(g1s));
disp('error coefficients');
disp(errorg(j));


% computation of errors for MPM
disp('Results obtained with MPM ... ');
disp(j);

ym=statio(gm,pol1,x);  % exponential sum reconstructed with MPM
errorfm(j)=norm(y-ym,inf)./max(y);
disp('error function');
disp(errorfm(j));

realpol1=sort(real(pol1));
imagpol1=sort(imag(pol1));
errorrealknots_MPM(j)=norm(realknots-realpol1,inf);
errorimagknots_MPM(j)=norm(imagknots-imagpol1,inf);
disp('error knots (real part and imaginary part)');
disp(errorrealknots_MPM(j));
disp(errorimagknots_MPM(j));

gms=sort(gm);
errorgm(j)=norm(gms-g1s,inf)./max(abs(g1s));
disp('error coefficients');
disp(errorgm(j));

% Plot knots
figure
sz=100;
scatter(real(knots),imag(knots),sz,'g','filled');
hold on
sz=40;
scatter(real(pol),imag(pol),sz,'r','+');
hold on
sz=40;
scatter(real(pol1),imag(pol1),sz,'b','o');
title('Knots: green - original signal, red - ESPIRA II, blue - MPM')
hold off

f1=f(1:51);
z1=0:50;

% plot original signal and signal reconstructed with ESPIRA II
figure
abb=plot(x,real(y),'black');
set(abb,'linewidth',2); set(gca,'fontsize',18); %print -depsc bsp22.eps;
axis([0,50,-20,40])
hold on 
scatter(z1,real(f1),'r','filled');
hold on
abb1=plot(x,real(y1),'magenta');
set(abb1,'linewidth',2); %set(gca,'fontsize',18);
title({'Black - original signal, red dots - noisy samples'; 'magenta - signal reconstructed with ESPIRA II'})
hold off
 
% plot original signal and signal reconstructed with MPM
figure
abb=plot(x,real(y),'black');
set(abb,'linewidth',2); set(gca,'fontsize',18); %print -depsc bsp22.eps;
axis([0,50,-20,40])
hold on 
scatter(z1,real(f1),'r','filled')
hold on
abb2=plot(x,real(ym),'blue');
set(abb2,'linewidth',2);
title({'Black - original signal, red dots - noisy samples';'blue - signal reconstructed with MPM'})
hold off

end


% Average errors for anz>1
disp('Average errors')
error_ESPIRA_real_knots_av=norm(errorrealknots_ESPIRA,1)/anz
error_ESPIRA_imag_knots_av=norm(errorimagknots_ESPIRA,1)/anz
error_ESPIRA_coef_av=norm(errorg,1)/anz
error_ESPIRA_func_av=norm(errorf,1)/anz

error_MPM_real_knots_av=norm(errorrealknots_MPM,1)/anz
error_MPM_imag_knots_av=norm(errorimagknots_MPM,1)/anz
error_MPM_coef_av=norm(errorgm,1)/anz
error_MPM_func_av=norm(errorfm,1)/anz

% Max errors for anz>1
disp('Max errors')
error_ESPIRA_real_knots_max=norm(errorrealknots_ESPIRA,inf)
error_ESPIRA_imag_knots_max=norm(errorimagknots_ESPIRA,inf)
error_ESPIRA_coef_max=norm(errorg,inf)
error_ESPIRA_func_max=norm(errorf,inf)

error_MPM_real_knots_max=norm(errorrealknots_MPM,inf)
error_MPM_imag_knots_max=norm(errorimagknots_MPM,inf)
error_MPM_coef_max=norm(errorgm,inf)
error_MPM_func_max=norm(errorfm,inf)

% Min errors for anz>1
disp('Min errors')
error_ESPIRA_real_knots_min=min(errorrealknots_ESPIRA)
error_ESPIRA_imag_knots_min=min(errorimagknots_ESPIRA)
error_ESPIRA_coef_min=min(errorg)
error_ESPIRA_func_min=min(errorf)

error_MPM_real_knots_min=min(errorrealknots_MPM)
error_MPM_imag_knots_min=min(errorimagknots_MPM)
error_MPM_coef_min=min(errorgm)
error_MPM_func_min=min(errorfm)

function f=statio(g1,knot1,x)
[N,~]=size(g1);
f=0;
for k=1:N
f= f+ g1(k)*knot1(k).^x;
end
end






